import 'dart:math';

import 'package:flutter/material.dart';

import 'package:sudoku/entities/cell.dart';
import 'package:sudoku/layout/board.dart';
import 'package:sudoku/provider/data.dart';
import 'package:sudoku/utils/board_utils.dart';
import 'package:sudoku/utils/game_utils.dart';

class Game {
  static Widget buildGameWidget(Data myProvider) {
    final bool gameIsFinished = BoardUtils.checkBoardIsSolved(myProvider);

    return Column(
      mainAxisAlignment: MainAxisAlignment.start,
      crossAxisAlignment: CrossAxisAlignment.center,
      children: [
        Board.buildGameBoard(myProvider),
        const SizedBox(height: 2),
        gameIsFinished
            ? Game.buildEndGameMessage(myProvider)
            : Game.buildSelectCellValueBar(myProvider),
      ],
    );
  }

  static Container buildSelectCellValueBar(Data myProvider) {
    final List<List<Cell>> cells = myProvider.cells;

    final bool isCellSelected =
        (myProvider.currentCellCol != null && myProvider.currentCellRow != null);
    final bool isUpdatableCellSelected = isCellSelected
        ? !cells[myProvider.currentCellRow ?? 0][myProvider.currentCellCol ?? 0].isFixed
        : false;
    final int maxValue = myProvider.blockSizeHorizontal * myProvider.blockSizeVertical;

    const int maxItemsPerLine = 10;
    final int linesCount = (maxValue / maxItemsPerLine).floor() + 1;
    final int itemsCountPerLine = min(maxItemsPerLine, maxValue + 1);

    return Container(
      margin: const EdgeInsets.all(2),
      padding: const EdgeInsets.all(2),
      child: Table(
        defaultColumnWidth: const IntrinsicColumnWidth(),
        children: [
          for (int lineIndex = 0; lineIndex < linesCount; lineIndex++)
            TableRow(
              children: [
                for (int value = lineIndex * itemsCountPerLine;
                    value < (lineIndex + 1) * itemsCountPerLine;
                    value++)
                  Column(
                    children: [
                      Cell(isUpdatableCellSelected ? (value <= maxValue ? value : -1) : -1,
                              false)
                          .widgetUpdateValue(myProvider)
                    ],
                  ),
              ],
            ),
        ],
      ),
    );
  }

  static Widget buildRestartGameButton(Data myProvider) {
    return TextButton(
      child: const Image(
        image: AssetImage('assets/icons/button_back.png'),
        fit: BoxFit.fill,
      ),
      onPressed: () => GameUtils.quitGame(myProvider),
    );
  }

  static Container buildEndGameMessage(Data myProvider) {
    const Image decorationImage = Image(
      image: AssetImage('assets/icons/game_win.png'),
      fit: BoxFit.fill,
    );

    return Container(
      margin: const EdgeInsets.all(2),
      padding: const EdgeInsets.all(2),
      child: Table(
        defaultColumnWidth: const IntrinsicColumnWidth(),
        children: [
          TableRow(
            children: [
              const Column(
                children: [decorationImage],
              ),
              Column(
                children: [
                  myProvider.animationInProgress
                      ? decorationImage
                      : buildRestartGameButton(myProvider)
                ],
              ),
              const Column(
                children: [decorationImage],
              ),
            ],
          ),
        ],
      ),
    );
  }
}
